

#undef min
#undef max
#undef map

#include <list>
#include <map>

using namespace std;

class RadioShuttle
{
public:
    typedef uint32_t  devid_t;

private:
    enum PacketStatus {
        PS_Queued,
        PS_Sent,
        PS_GotSendSlot,
        PS_WaitForConfirm,
        PS_SendRequestCompleted,
        PS_SendRequestConfirmed,
        PS_SendTimeout,
    };
    enum RadioType {
        RS_Node_Offline,     // Sleep mode until sending, < 10k RAM
        RS_Node_Checking,   // Sleep mode, checks for messages regulary, < 10k RAM
        RS_Node_Online,     // Always powered-on, < 10k RAM
        RS_Station_Basic, // Always active for one or more apps < 15k RAM
        RS_Station_Server,  // Always active, lots of memory, routing options, < 1 MB RAM
    };
    struct SignalStrengthEntry {
        int rx_dBm;
        devid_t stationID;
        time_t lastUpdate;
        int rcnCnt;
    };
    struct RadioEntry; // forward decl.
    struct ReceivedMsgEntry {
        void *RxData;
        int RxSize;
        int rssi;
        int snr;
        struct RadioEntry *re;
    };
    struct RadioEntry {
        volatile signed char _CADdetected;
        uint16_t lastTxSize;
        int lastTxPower;
        int timeOnAir12Bytes;
        int maxTimeOnAir;
        int retry_ms;
        uint32_t lastTxDone;
        volatile bool txDoneReceived;
        uint32_t random;
        uint32_t random2;
    };
    struct AppEntry {
        int AppID;
        int msgID;
        void *password;
        uint8_t pwLen;
        bool pwdConnected;
    };
    struct TimeOnAirSlotEntry {
        devid_t stationID;
        int AppID;
        uint8_t channel;
        uint8_t factor;
        uint32_t busy_time;
        int busy_ms;
    };
    struct ConnectEntry {
        devid_t stationID;
        int AppID;
        bool authorized;
        uint32_t random[2];
    };
    struct SendMsgEntry {
        int AppID;
        void *data;
        int len;
        int flags;
        devid_t stationID;
        int msgID;
        int retryCount;
        
        AppEntry *aep;
        ConnectEntry *cep;
        PacketStatus pStatus;
        int respWindow;
        uint32_t responseTime;
        uint32_t lastSentTime;
        uint32_t lastTimeOnAir;
        uint32_t confirmTimeout;
        int retry_ms;
        uint8_t channel;
        uint8_t factor;
        uint32_t securityData[8];
        uint32_t tmpRandom[2];
    };

private:
    const char *_name;
    devid_t _deviceID;
    devid_t _tmpdeviceID;
    uint8_t _uuid[16];
    RadioType _radioType;
    int _maxMTUSize;
    list<RadioEntry> _radios;
    map<int, AppEntry> _apps;
    map<std::pair<devid_t,int>, ConnectEntry> _connections;
    list<SendMsgEntry> _sends;
    list<ReceivedMsgEntry> _recvs;
    map<devid_t, SignalStrengthEntry> _signals;
    list<TimeOnAirSlotEntry> _airtimes;
    volatile uint32_t prevWakeup;
    int SetTimerCount;
};



void setup() {
  // put your setup code here, to run once:

}

void loop() {
  // put your main code here, to run repeatedly:

}
